<#assign import_block>
import ${srfclassname(ctrl.codeName)}Model from './${srffilepath2(ctrl.codeName)}-${ctrl.getControlType()?lower_case}-model';
</#assign>
<script lang='tsx'>
import { Prop, Provide, Emit, Watch, Model } from 'vue-property-decorator';
import { CreateElement } from 'vue';
import { Subject, Subscription } from 'rxjs';
import { ControlInterface } from '@/interface/control';
import { AppMenusCtrlBase } from '@/crm-core';
<#if appde??>
import ${srfclassname('${appde.getCodeName()}')}Service from '@/service/${srffilepath2(appde.getCodeName())}/${srffilepath2(appde.getCodeName())}-service';
</#if>
import ${srfclassname(ctrl.codeName)}Service from './${srffilepath2(ctrl.codeName)}-${ctrl.getControlType()?lower_case}-service';

<#if ctrl.getPSUIActions?? && ctrl.getPSUIActions()??>
<#list ctrl.getPSUIActions() as uiAction>
<#if uiAction.getPSAppDataEntity()??>
<#assign curAppEntity = uiAction.getPSAppDataEntity()/>
<#if !P.exists("importService", curAppEntity.getId(), "")>
import ${srfclassname('${curAppEntity.getCodeName()}')}UIService from '@/uiservice/${srffilepath2(curAppEntity.getCodeName())}/${srffilepath2(curAppEntity.getCodeName())}-ui-service';
</#if>
</#if>
</#list>
</#if>
<#if import_block??>${import_block}</#if>

/**
 * ${ctrl.getLogicName()}基类
 *
 * @export
 * @class ${srfclassname(ctrl.codeName)}Base
 * @extends {AppMenusCtrlBase}
 */
export default class ${srfclassname(ctrl.codeName)}Base extends AppMenusCtrlBase {

    /**
     * 建构部件服务对象
     *
     * @type {${srfclassname(ctrl.codeName)}Service}
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    public service: ${srfclassname(ctrl.codeName)}Service = new ${srfclassname(ctrl.codeName)}Service({ $store: this.$store });

    /**
     * 菜单对齐方向
     *
     * @type {CrmIndexService}
     * @memberof CrmIndex
     */
    protected menuAlign: string = '<#if view.getMainMenuAlign?? && view.getMainMenuAlign() ??>${view.getMainMenuAlign()}</#if>';
<#if appde??>

    /**
     * 实体服务对象
     *
     * @type {${srfclassname('${appde.getCodeName()}')}Service}
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    public appEntityService: ${srfclassname('${appde.getCodeName()}')}Service = new ${srfclassname('${appde.getCodeName()}')}Service({ $store: this.$store });
</#if>
<#if ctrl.getPSAppViewLogics?? && ctrl.getPSAppViewLogics()??>
<#list ctrl.getPSAppViewLogics() as logic>
<#if logic.getLogicTrigger() == "CUSTOM" || logic.getLogicTrigger() == "CTRLEVENT">

${P.getLogicCode(logic, "LOGIC.vue").code}
</#if>
</#list>
</#if>
<#if ctrl.getPSUIActions?? && ctrl.getPSUIActions()??>
<#list ctrl.getPSUIActions() as uiAction>
<#if !uiAction.getPSAppDataEntity()??>

${P.getLogicCode(uiAction, "LOGIC.vue").code}
</#if>
</#list>
</#if>

    /**
     * 菜单模型
     *
     * @protected
     * @type {${srfclassname(ctrl.codeName)}Model}
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    protected menuModel: ${srfclassname(ctrl.codeName)}Model = new ${srfclassname(ctrl.codeName)}Model();

    /**
     * 触发方式，默认click
     *
     * @type {string}
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    @Provide()
    public trigger: string = 'click';
    <#if ctrl.getPSSysCounter?? && ctrl.getPSSysCounter()??>
    <#assign sysCounter = ctrl.getPSSysCounter()/>

    /**
     * 计数器对象
     *
     * @type {(UICounter | null)}
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    public ${sysCounter.getCodeName()}: UICounter | null = null;
    </#if>
<#if ctrl.getPSSysCounter?? && ctrl.getPSSysCounter()??>

    /**
     * vue  生命周期
     *
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    public created() {
        <#assign sysCounter = ctrl.getPSSysCounter()/>
        this.${sysCounter.getCodeName()} = new UICounter({
            name: '${sysCounter.getCodeName()}',
            counterId: '${sysCounter.id}',
            timer: ${sysCounter.timer?c},
            url: ``,
        });
        this.${sysCounter.getCodeName()}.uiCounterData.subscribe((data: any) => {
            this.counterdata = {};
            Object.assign(this.counterdata, data);
        });
        super.created();
    }
</#if>
<#if ctrl.getPSSysCounter?? && ctrl.getPSSysCounter()??>

    /**
     * vue 生命周期
     *
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    public destroyed() {
        super.destroyed();
        <#assign sysCounter = ctrl.getPSSysCounter()/>
        if(this.${sysCounter.getCodeName()}) {
            this.${sysCounter.getCodeName()}.uiCounterData.unsubscribe();
            this.${sysCounter.getCodeName()}.close();
        }
    }
</#if>

    /**
     * 菜单点击
     *
     * @protected
     * @param {*} item 菜单数据
     * @memberof ${srfclassname(ctrl.codeName)}
     */
    protected click(item: any) {
        if (item) {
            switch (item.appfunctag) {
                <#if ctrl.getPSAppFuncs()??>
                <#assign appFuncs = ctrl.getPSAppFuncs()>
                <#list appFuncs as singFuncs>
                case '${singFuncs.getCodeName()}': 
                    this.click${singFuncs.codeName}(item);
                    return;
                </#list>
                </#if>
                default:
                    console.warn('未指定应用功能');
            }
        }
    }
<#if ctrl.getPSAppFuncs()??>
<@ibizindent blank=4>
<#list ctrl.getPSAppFuncs() as singleFuncs>

${P.getLogicCode(singleFuncs,"LOGIC.vue").code}
</#list>
</@ibizindent>
</#if>

    public render(): any {
        return (
            <div class="app-app-menu">
                <app-menus menuAlign={this.menuAlign} menus={this.menus} on-menu-click={(item: any) => this.click(item)}/>
            </div>
        );
    }
    
<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>